-- Game class definition

Game = class("Game")

function Game:initialize()

	self.world = love.physics.newWorld(0, 0, true)
	self.world:setCallbacks(
		Game.beginContact,
		Game.endContact,
		Game.preSolve,
		Game.postSolve
	)
	
	self.entityManager = EntityManager(self.world)
	
	self.entityManager:setPlayer(
		Player(
			self.world,
			Point(SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2),
			self.entityManager
		)
	)
	
	self:createBoundaryWalls()
	
	self.vignetteFilter = love.graphics.newImage("res/gfx/vignette.png")
	
	if SHADERS then

		self.canvas = love.graphics.newCanvas()
	
	end
	
	self.camera = Camera(Point(0, 0), 0, SCALE)
	
	self.currentWave = Wave(self.world, self.entityManager)
	
	self.done = false
	
end

function Game:createBoundaryWalls()

	-- top wall
	self.entityManager:addEntity(
		Wall(
			self.world,
			Point(0, -1),
			Point(SCREEN_WIDTH, 1),
			self.entityManager
		)
	)
	
	-- bottom wall
	self.entityManager:addEntity(
		Wall(
			self.world,
			Point(0, SCREEN_HEIGHT),
			Point(SCREEN_WIDTH, 1),
			self.entityManager
		)
	)
	
	-- player walls will only stop the player from going through
	
	-- left wall
	self.entityManager:addEntity(
		PlayerWall(
			self.world,
			Point(0, -1),
			Point(1, SCREEN_HEIGHT),
			self.entityManager
		)
	)
	
	-- right wall
	self.entityManager:addEntity(
		PlayerWall(
			self.world,
			Point(SCREEN_WIDTH, 0),
			Point(1, SCREEN_HEIGHT),
			self.entityManager
		)
	)
	
	-- kill wall will destroy any enemies that slip by the player
	self.entityManager:addEntity(
		KillWall(
			self.world,
			Point(-200, 0),
			Point(1, SCREEN_HEIGHT),
			self.entityManager
		)
	)

end

function Game:update(dt)

	self.world:update(dt)
	self.currentWave:update(dt)
	self.entityManager:update(dt)

end

function Game:draw()

	if self.done then
	
		if SHADERS then
		
			love.graphics.setCanvas(self.canvas)
		
			self:drawBackground()
			
			love.graphics.setColor(COLOR.BLACK)
			
			love.graphics.printf(
				self.entityManager:getPlayer().score,
				0,
				(SCREEN_HEIGHT / 2) - 68,
				SCREEN_WIDTH,
				"center"
			)
			
			love.graphics.printf(
				"press \"r\"",
				0,
				(SCREEN_HEIGHT / 2) + 20,
				SCREEN_WIDTH,
				"center"
			)
			
			love.graphics.setCanvas()
			
			SEPERATOR_SHADER:send("textureSize", {SCREEN_WIDTH, SCREEN_HEIGHT})
			
			love.graphics.setPixelEffect(SEPERATOR_SHADER)
			love.graphics.draw(self.canvas)
			love.graphics.setPixelEffect()
			
			self.canvas:clear()
			
		else
	
			self:drawBackground()
			
			love.graphics.setColor(COLOR.BLACK)
			
			love.graphics.printf(
				self.entityManager:getPlayer().score,
				0,
				(SCREEN_HEIGHT / 2) - 68,
				SCREEN_WIDTH,
				"center"
			)
			
			love.graphics.printf(
				"press \"r\"",
				0,
				(SCREEN_HEIGHT / 2) + 20,
				SCREEN_WIDTH,
				"center"
			)
			
		end
		
		if love.keyboard.isDown("r") then
			
			self:initialize()
			
		end
	
	else

		self.camera:bind()
		
		if SHADERS then
		
			love.graphics.setCanvas(self.canvas)
		
			self:drawBackground()
		
			self.entityManager:draw()
			
			self:drawCursor()
			
			love.graphics.setCanvas()
			
			SEPERATOR_SHADER:send("textureSize", {SCREEN_WIDTH, SCREEN_HEIGHT})
			
			love.graphics.setPixelEffect(SEPERATOR_SHADER)
			love.graphics.draw(self.canvas)
			love.graphics.setPixelEffect()
			
			self.canvas:clear()
			
		else
		
			self:drawBackground()
			self.entityManager:draw()
			self:drawCursor()
		
		end
		
		self.camera:unbind()
	
	end

end

function Game:drawBackground()
	
	love.graphics.setColor(COLOR.GREY)
	love.graphics.rectangle("fill", 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT)
	love.graphics.draw(
		self.vignetteFilter,
		0,
		0,
		0,
		SCREEN_WIDTH / self.vignetteFilter:getWidth(),
		SCREEN_HEIGHT / self.vignetteFilter:getHeight()
	)
	
end

function Game:drawCursor()

	local mx, my = love.mouse.getPosition()
	
	love.graphics.setColor(COLOR.FRIEND)
	love.graphics.circle("line", mx, my, 2, 8)

end

function Game.beginContact(a, b, col)

	aUserData = a:getUserData()
	bUserData = b:getUserData()
	
	if aUserData.id == "player" then
	
		if bUserData.id == "enemy" then
		
			aUserData.class:kill()
		
		end
	
	elseif bUserData.id == "player" then
	
		if aUserData.id == "enemy" then
		
			bUserData.class:kill()
		
		end
	
	end
	
	if aUserData.id == "killWall" then
	
		
		if bUserData.id == "enemy" then
		
			bUserData.class.score = 0
		
		end
		
		bUserData.class:kill()
	
	elseif bUserData.id == "killWall" then
	
		if aUserData.id == "enemy" then
		
			aUserData.class.score = 0
		
		end
	
		aUserData.class:kill()
	
	end
	
	if aUserData.id == "bullet" then
	
		if aUserData.friendly then

			if bUserData.id == "enemy" then

				bUserData.class:hurt(aUserData.class.damage)

			end

		else

			if bUserData.id == "player" then

				bUserData.class:hurt(aUserData.class.damage)

			end

		end
		
		-- bullets don't kill eachother!
		if bUserData.id ~= "bullet" then
		
			aUserData.class:kill()
			
		end
		
	elseif bUserData.id == "bullet" then
	
		if bUserData.friendly then

			if aUserData.id == "enemy" then

				aUserData.class:hurt(bUserData.class.damage)

			end

		else

			if aUserData.id == "player" then

				aUserData.class:hurt(bUserData.class.damage)
				
			end

		end

		-- bullets don't kill eachother!
		if aUserData.id ~= "bullet" then
		
			bUserData.class:kill()
			
		end
	
	end

end

function Game.endContact(a, b, col)

end

function Game.preSolve(a, b, col)

end

function Game.postSolve(a, b, col)

end